@@ -128,7 +128,7 @@ update occurred, not counting content."
 
 (defun elfeed-db-set-update-time ()
   "Update the database last-update time."
-  (plist-put elfeed-db :last-update (float-time))
+  (setf elfeed-db (plist-put elfeed-db :last-update (float-time)))
   (run-hooks 'elfeed-db-update-hook))
 
 (defun elfeed-db-add (entries)
@@ -169,18 +169,34 @@ update occurred, not counting content."
   (let ((all (apply #'append tags (nconc more-tags (list ())))))
     (cl-delete-duplicates (cl-sort all #'string< :key #'symbol-name))))
 
-(defun elfeed-tag (entry &rest tags)
+(defun elfeed-tag-1 (entry &rest tags)
   "Add TAGS to ENTRY."
   (let ((current (elfeed-entry-tags entry)))
     (setf (elfeed-entry-tags entry)
           (elfeed-normalize-tags (append tags current)))))
 
-(defun elfeed-untag (entry &rest tags)
+(defun elfeed-untag-1 (entry &rest tags)
   "Remove TAGS from ENTRY."
   (setf (elfeed-entry-tags entry)
         (cl-loop for tag in (elfeed-entry-tags entry)
                  unless (memq tag tags) collect tag)))
 
+(defun elfeed-tag (entry-or-entry-list &rest tags)
+  "Add TAGS to ENTRY-OR-ENTRY-LIST and run `elfeed-tag-hooks'."
+  (let* ((entries (if (elfeed-entry-p entry-or-entry-list)
+                      (list entry-or-entry-list)
+                    entry-or-entry-list)))
+    (run-hook-with-args 'elfeed-tag-hooks entries tags)
+    (cl-loop for entry in entries do (apply #'elfeed-tag-1 entry tags))))
+
+(defun elfeed-untag (entry-or-entry-list &rest tags)
+  "Remove TAGS from ENTRY-OR-ENTRY-LIST and run `elfeed-untag-hooks'."
+  (let* ((entries (if (elfeed-entry-p entry-or-entry-list)
+                      (list entry-or-entry-list)
+                    entry-or-entry-list)))
+    (run-hook-with-args 'elfeed-untag-hooks entries tags)
+    (cl-loop for entry in entries do (apply #'elfeed-untag-1 entry tags))))
+
 (defun elfeed-tagged-p (tag entry)
   "Return true if ENTRY is tagged by TAG."
   (memq tag (elfeed-entry-tags entry)))
@@ -248,7 +264,7 @@ The FEED-OR-ID may be a feed struct or a feed ID (url)."
 (defun elfeed-db-save ()
   "Write the database index to the filesystem."
   (elfeed-db-ensure)
-  (plist-put elfeed-db :version elfeed-db-version)
+  (setf elfeed-db (plist-put elfeed-db :version elfeed-db-version))
   (mkdir elfeed-db-directory t)
   (let ((coding-system-for-write 'utf-8))
     (with-temp-file (expand-file-name "index" elfeed-db-directory)
@@ -275,7 +291,7 @@ The FEED-OR-ID may be a feed struct or a feed ID (url)."
           (setf (elfeed-entry-id entry) new-id
                 (gethash new-id elfeed-db-entries) entry)
           (avl-tree-enter elfeed-db-index new-id)))))
-  (plist-put elfeed-db :version elfeed-db-version)
+  (setf elfeed-db (plist-put elfeed-db :version elfeed-db-version))
   elfeed-db-version)
 
 (defun elfeed-db-load ()
@@ -283,11 +299,12 @@ The FEED-OR-ID may be a feed struct or a feed ID (url)."
   (let ((index (expand-file-name "index" elfeed-db-directory))
         (enable-local-variables nil)) ; don't set local variables from index!
     (if (not (file-exists-p index))
-        (let ((db (setf elfeed-db (list :version elfeed-db-version))))
-          (plist-put db :feeds (make-hash-table :test 'equal))
-          (plist-put db :entries (make-hash-table :test 'equal))
-          ;; Compiler will warn about this (bug#15327):
-          (plist-put db :index (avl-tree-create #'elfeed-db-compare)))
+        (setf elfeed-db
+              `(:version ,elfeed-db-version
+                :feeds ,(make-hash-table :test 'equal)
+                :entries ,(make-hash-table :test 'equal)
+                ;; Compiler may warn about this (bug#15327):
+                :index ,(avl-tree-create #'elfeed-db-compare)))
       (with-current-buffer (find-file-noselect index :nowarn)
         (goto-char (point-min))
         (setf elfeed-db (read (current-buffer)))
@@ -300,6 +317,15 @@ The FEED-OR-ID may be a feed struct or a feed ID (url)."
     (when (version< (or (plist-get elfeed-db :version) "0") elfeed-db-version)
       (elfeed-db-upgrade))))
 
+(defun elfeed-db-unload ()
+  "Unload the database so that it can be operated on externally."
+  (interactive)
+  (elfeed-db-save)
+  (setf elfeed-db nil
+        elfeed-db-feeds nil
+        elfeed-db-entries nil
+        elfeed-db-index nil))
+
 (defun elfeed-db-ensure ()
   "Ensure that the database has been loaded."
   (when (null elfeed-db) (elfeed-db-load)))
@@ -398,7 +424,8 @@ The FEED-OR-ID may be a feed struct or a feed ID (url)."
       ref
     (let ((index (and (hash-table-p elfeed-ref-archive)
                       (gethash (elfeed-ref-id ref) elfeed-ref-archive)))
-          (archive-file (elfeed-ref-archive-filename ".gz")))
+          (archive-file (elfeed-ref-archive-filename ".gz"))
+          (coding-system-for-read 'utf-8))
       (if (and index (file-exists-p archive-file))
           (progn
             (when (null elfeed-ref-cache)
