@@ -9,12 +9,12 @@
 (require 'wid-edit) ; widget-inactive face
 (require 'bookmark)
 
-(provide 'elfeed-search)
-
 (require 'elfeed)
 (require 'elfeed-db)
 (require 'elfeed-lib)
-(require 'elfeed-show)
+
+;; Interface to elfeed-show (lazy required)
+(declare-function elfeed-show-entry 'elfeed-show (entry))
 
 (defvar elfeed-search-entries ()
   "List of the entries currently on display.")
@@ -189,9 +189,10 @@ When live editing the filter, it is bound to :live.")
   (make-local-variable 'elfeed-search-entries)
   (make-local-variable 'elfeed-search-filter)
   (add-hook 'elfeed-update-hooks #'elfeed-search-update)
+  (add-hook 'elfeed-update-init-hooks #'elfeed-search-update--force)
   (add-hook 'kill-buffer-hook #'elfeed-db-save t t)
   (elfeed-search-update :force)
-  (run-hooks 'elfeed-search-mode-hook))
+  (run-mode-hooks 'elfeed-search-mode-hook))
 
 (defun elfeed-search-buffer ()
   (get-buffer-create "*elfeed-search*"))
@@ -342,6 +343,47 @@ The customization `elfeed-search-date-format' sets the formatting."
                                (push element matches)))))
     (list after must-have must-not-have matches not-matches limit)))
 
+(defun elfeed-search--recover-units (seconds)
+  "Pick a reasonable filter representation for SECONDS."
+  (let ((units '((60   1 "minute")
+                 (60   1 "hour")
+                 (24   1 "day")
+                 (7    1 "week")
+                 (30   7 "month")
+                 (1461 120 "year")))
+        (value (float seconds))
+        (name "second"))
+    (cl-loop for (n d unit) in units
+             for next-value = (/ (* value d) n)
+             when (< next-value 1.0)
+             return t
+             do (setf name unit
+                      value next-value))
+    (let ((count (format "%.4g" value)))
+      (format "@%s-%s%s-ago" count name (if (equal count "1") "" "s")))))
+
+(defun elfeed-search-unparse-filter (filter)
+  "Inverse of `elfeed-search-parse-filter', returning a string.
+
+The time (@n-units-ago) filter may not exactly match the
+original, but will be equal in its effect."
+  (let ((output ()))
+    (cl-destructuring-bind
+        (after must-have must-not-have matches not-matches limit) filter
+      (when after
+        (push (elfeed-search--recover-units after) output))
+      (dolist (tag must-have)
+        (push (format "+%S" tag) output))
+      (dolist (tag must-not-have)
+        (push (format "-%S" tag) output))
+      (dolist (re matches)
+        (push re output))
+      (dolist (re not-matches)
+        (push (concat "!" re) output))
+      (when limit
+        (push (format "#%d" limit) output))
+      (mapconcat #'identity (nreverse output) " "))))
+
 (defun elfeed-search-filter (filter entry feed &optional count)
   "Return non-nil if ENTRY and FEED pass FILTER.
 
@@ -558,9 +600,11 @@ Given a prefix, this function becomes `elfeed-search-fetch-visible'."
   (let ((n (cl-position entry elfeed-search-entries)))
     (when n (elfeed-search-update-line (+ elfeed-search--offset n)))))
 
-(defun elfeed-search-selected (&optional ignore-region)
-  "Return a list of the currently selected feeds."
-  (let ((use-region (and (not ignore-region) (use-region-p))))
+(defun elfeed-search-selected (&optional ignore-region-p)
+  "Return a list of the currently selected feeds.
+
+If IGNORE-REGION-P is non-nil, only return the entry under point."
+  (let ((use-region (and (not ignore-region-p) (use-region-p))))
     (let ((start (if use-region (region-beginning) (point)))
           (end   (if use-region (region-end)       (point))))
       (cl-loop for line from (line-number-at-pos start)
@@ -568,7 +612,9 @@ Given a prefix, this function becomes `elfeed-search-fetch-visible'."
                for offset = (- line elfeed-search--offset)
                when (and (>= offset 0) (nth offset elfeed-search-entries))
                collect it into selected
-               finally (return (if ignore-region (car selected) selected))))))
+               finally (return (if ignore-region-p
+                                   (car selected)
+                                 selected))))))
 
 (defun elfeed-search-browse-url (&optional use-generic-p)
   "Visit the current entry in your browser using `browse-url'.
@@ -592,7 +638,7 @@ browser defined by `browse-url-generic-program'."
          (links (mapcar #'elfeed-entry-link entries))
          (links-str (mapconcat #'identity links " ")))
     (when entries
-      (cl-loop for entry in entries do (elfeed-untag entry 'unread))
+      (elfeed-untag entries 'unread)
       (kill-new links-str)
       (if (fboundp 'gui-set-selection)
           (gui-set-selection elfeed-search-clipboard-type links-str)
@@ -606,7 +652,7 @@ browser defined by `browse-url-generic-program'."
   "Apply TAG to all selected entries."
   (interactive (list (intern (read-from-minibuffer "Tag: "))))
   (let ((entries (elfeed-search-selected)))
-    (cl-loop for entry in entries do (elfeed-tag entry tag))
+    (elfeed-tag entries tag)
     (mapc #'elfeed-search-update-entry entries)
     (unless (use-region-p) (forward-line))))
 
@@ -614,24 +660,27 @@ browser defined by `browse-url-generic-program'."
   "Remove TAG from all selected entries."
   (interactive (list (intern (read-from-minibuffer "Tag: "))))
   (let ((entries (elfeed-search-selected)))
-    (cl-loop for entry in entries do (elfeed-untag entry tag))
+    (elfeed-untag entries tag)
     (mapc #'elfeed-search-update-entry entries)
     (unless (use-region-p) (forward-line))))
 
 (defun elfeed-search-toggle-all (tag)
   "Toggle TAG on all selected entries."
   (interactive (list (intern (read-from-minibuffer "Tag: "))))
-  (let ((entries (elfeed-search-selected)))
+  (let ((entries (elfeed-search-selected)) entries-tag entries-untag)
     (cl-loop for entry in entries
              when (elfeed-tagged-p tag entry)
-             do (elfeed-untag entry tag)
-             else do (elfeed-tag entry tag))
+             do (push entry entries-untag)
+             else do (push entry entries-tag))
+    (elfeed-tag entries-tag tag)
+    (elfeed-untag entries-untag tag)
     (mapc #'elfeed-search-update-entry entries)
     (unless (use-region-p) (forward-line))))
 
 (defun elfeed-search-show-entry (entry)
   "Display the currently selected item in a buffer."
   (interactive (list (elfeed-search-selected :ignore-region)))
+  (require 'elfeed-show)
   (when (elfeed-entry-p entry)
     (elfeed-untag entry 'unread)
     (elfeed-search-update-entry entry)
@@ -742,4 +791,6 @@ Sets the :title key of the feed's metadata. See `elfeed-meta'."
 (add-to-list 'desktop-buffer-mode-handlers
              '(elfeed-search-mode . elfeed-search-desktop-restore))
 
+(provide 'elfeed-search)
+
 ;;; elfeed-search.el ends here
