@@ -10,8 +10,23 @@
 ;; Elfeed is a web feed client for Emacs, inspired by notmuch. See
 ;; the README for full documentation.
 
+;; Notice: Before stamping a new release the following places need to
+;; be updated:
+;; * elfeed.el (`elfeed-version')
+;; * elfeed-pkg.el
+;; * web/elfeed-web-pkg.el
+;; * Makefile (VERSION)
+
 ;;; History:
 
+;; Version 2.2.0:
+;;   * Support for org links (elfeed-link.el)
+;;   * Added `elfeed-db-unload'
+;;   * New `elfeed-curl-retrieve' interface (breaking changes)
+;;   * New hooks `elfeed-tag-hooks' and `elfeed-untag-hooks'
+;; Version 2.1.1: fixes and some features
+;;   * Added `elfeed-show-entry-author' customization variable.
+;;   * Added `elfeed-search-unparse-filter'.
 ;; Version 2.1.0: features and new database format
 ;;   * New entry ID based only on domain, not whole feed
 ;;   * Byte-compiled search filters (`elfeed-search-compile-filter')
@@ -111,14 +126,21 @@
 (require 'xml-query)
 (require 'url-parse)
 (require 'url-queue)
+
+(require 'elfeed-db)
+(require 'elfeed-lib)
 (require 'elfeed-log)
 (require 'elfeed-curl)
 
+;; Interface to elfeed-search (lazy required)
+(declare-function elfeed-search-buffer 'elfeed-search ())
+(declare-function elfeed-search-mode   'elfeed-search ())
+
 (defgroup elfeed ()
   "An Emacs web feed reader."
   :group 'comm)
 
-(defconst elfeed-version "2.1.0")
+(defconst elfeed-version "2.2.0")
 
 (defcustom elfeed-feeds ()
   "List of all feeds that Elfeed should follow.
@@ -162,13 +184,6 @@ Each function should accept no arguments, and return a string or nil."
   :group 'elfeed
   :type 'string)
 
-(provide 'elfeed)
-
-(require 'elfeed-search)
-(require 'elfeed-lib)
-(require 'elfeed-db)
-(require 'elfeed-csv)
-
 (defcustom elfeed-initial-tags '(unread)
   "Initial tags for new entries."
   :group 'elfeed
@@ -192,6 +207,24 @@ It is called with 1 argument: the URL of the feed that was just
 updated. The hook is called even when no new entries were
 found.")
 
+(defvar elfeed-update-init-hooks ()
+  "Hooks called when one or more feed updates have begun.
+Receivers may want to, say, update a display to indicate that
+updates are pending.")
+
+(defvar elfeed-tag-hooks ()
+  "Hooks called when one or more entries add tags.
+It is called with 2 arguments. The first argument is the entry
+list. The second argument is the tag list.")
+
+(defvar elfeed-untag-hooks ()
+  "Hooks called when one or more entries remove tags.
+It is called with 2 arguments. The first argument is the entry
+list. The second argument is the tag list.")
+
+(defvar elfeed--inhibit-update-init-hooks nil
+  "When non-nil, don't run `elfeed-update-init-hooks'.")
+
 (defun elfeed-queue-count-active ()
   "Return the number of items in process."
   (if elfeed-use-curl
@@ -228,6 +261,11 @@ found.")
       elfeed-curl-timeout
     url-queue-timeout))
 
+(defun elfeed-is-status-error (status use-curl)
+  "Check if HTTP request returned status means a error."
+  (or (and use-curl (null status)) ; nil = error
+      (and (not use-curl) (eq (car status) :error))))
+
 (defmacro elfeed-with-fetch (url &rest body)
   "Asynchronously run BODY in a buffer with the contents from URL.
 This macro is anaphoric, with STATUS referring to the status from
@@ -245,7 +283,7 @@ of `elfeed-use-curl'."
              (push `("If-None-Match" . ,etag) headers))
            (when last-modified
              (push `("If-Modified-Since" . ,last-modified) headers))
-           (elfeed-curl-enqueue ,url cb headers))
+           (elfeed-curl-enqueue ,url cb :headers headers))
        (url-queue-retrieve ,url cb () t t))))
 
 (defun elfeed-unjam ()
@@ -260,7 +298,7 @@ This is a workaround for issues in `url-queue-retrieve'."
         (elfeed-log 'warn "Elfeed aborted feeds: %s"
                     (mapconcat #'identity fails " ")))
       (setf url-queue nil)))
-  (elfeed-search-update :force))
+  (run-hooks 'elfeed-update-init-hooks))
 
 ;; Parsing:
 
@@ -477,6 +515,12 @@ Only a list of strings will be returned."
                url-or-feed)))
     (mapcar #'elfeed-keyword->symbol (cdr (assoc url elfeed-feeds)))))
 
+(defun elfeed-apply-autotags-now ()
+  "Apply autotags to existing entries according to `elfeed-feeds'."
+  (interactive)
+  (with-elfeed-db-visit (entry feed)
+    (apply #'elfeed-tag entry (elfeed-feed-autotags feed))))
+
 (defun elfeed-handle-http-error (url status)
   "Handle an http error during retrieval of URL with STATUS code."
   (cl-incf (elfeed-meta (elfeed-db-get-feed url) :failures 0))
@@ -492,9 +536,10 @@ Only a list of strings will be returned."
 (defun elfeed-update-feed (url)
   "Update a specific feed."
   (interactive (list (completing-read "Feed: " (elfeed-feed-list))))
+  (unless elfeed--inhibit-update-init-hooks
+    (run-hooks 'elfeed-update-init-hooks))
   (elfeed-with-fetch url
-    (if (or (and use-curl (null status)) ; nil = error
-            (and (not use-curl) (eq (car status) :error)))
+    (if (elfeed-is-status-error status use-curl)
         (let ((print-escape-newlines t))
           (elfeed-handle-http-error
            url (if use-curl elfeed-curl-error-message status)))
@@ -554,8 +599,7 @@ Only a list of strings will be returned."
   (cl-pushnew url elfeed-feeds)
   (when (called-interactively-p 'any)
     (customize-save-variable 'elfeed-feeds elfeed-feeds))
-  (elfeed-update-feed url)
-  (elfeed-search-update :force))
+  (elfeed-update-feed url))
 
 ;;;###autoload
 (defun elfeed-update ()
@@ -563,8 +607,9 @@ Only a list of strings will be returned."
   (interactive)
   (elfeed-log 'info "Elfeed update: %s"
               (format-time-string "%B %e %Y %H:%M:%S %Z"))
-  (mapc #'elfeed-update-feed (elfeed--shuffle (elfeed-feed-list)))
-  (elfeed-search-update :force)
+  (let ((elfeed--inhibit-update-init-hooks t))
+    (mapc #'elfeed-update-feed (elfeed--shuffle (elfeed-feed-list))))
+  (run-hooks 'elfeed-update-init-hooks)
   (elfeed-db-save))
 
 ;;;###autoload
@@ -573,8 +618,7 @@ Only a list of strings will be returned."
   (interactive)
   (switch-to-buffer (elfeed-search-buffer))
   (unless (eq major-mode 'elfeed-search-mode)
-    (elfeed-search-mode))
-  (elfeed-search-update))
+    (elfeed-search-mode)))
 
 ;; New entry filtering
 
@@ -671,4 +715,11 @@ saved to your customization file."
                                 collect `(outline ((xmlUrl . ,url)
                                                    (title . ,title)))))))))))
 
+(provide 'elfeed)
+
+(cl-eval-when (load eval)
+  (require 'elfeed-csv)
+  (require 'elfeed-show)
+  (require 'elfeed-search))
+
 ;;; elfeed.el ends here
